// SIDLookup.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"

#pragma comment(lib, "msi")

BOOL APIENTRY DllMain(HMODULE hModule, 
                      DWORD  ul_reason_for_call, 
                      LPVOID lpReserved)
{
   switch (ul_reason_for_call)
   {
   case DLL_PROCESS_ATTACH:
      ::DisableThreadLibraryCalls(hModule);
      break;
   }
   return TRUE;
}

#define _countof(X) (sizeof(X) / sizeof(X[0]))

// helper for efficiently constructing well-known SIDs
struct SIDWithTwoSubauthorities {
	SID   base;
	DWORD subAuth;
};

struct SIDs 
{
   PCTSTR                     szName;
   SIDWithTwoSubauthorities   sid;
};

extern "C" UINT __stdcall
SIDLookup(MSIHANDLE hInstall)
{
   static const SIDs WellKnowns[] = 
   {
      { _T("SID_EVERYONE"),
         { SID_REVISION, 1, SECURITY_WORLD_SID_AUTHORITY, SECURITY_WORLD_RID, 0 } },
      { _T("SID_LOCAL"), 
         { SID_REVISION, 1, SECURITY_LOCAL_SID_AUTHORITY, SECURITY_LOCAL_RID, 0 } },
      { _T("SID_CREATOR_OWNER"), 
         { SID_REVISION, 1, SECURITY_CREATOR_SID_AUTHORITY, SECURITY_CREATOR_OWNER_RID, 0 } },
      { _T("SID_CREATOR_GROUP"), 
         { SID_REVISION, 1, SECURITY_CREATOR_SID_AUTHORITY, SECURITY_CREATOR_GROUP_RID, 0 } },
      { _T("SID_DIALUP"), 
         { SID_REVISION, 1, SECURITY_NT_AUTHORITY, SECURITY_DIALUP_RID, 0 } },
      { _T("SID_NETWORK"), 
         { SID_REVISION, 1, SECURITY_NT_AUTHORITY, SECURITY_NETWORK_RID, 0 } },
      { _T("SID_BATCH"), 
         { SID_REVISION, 1, SECURITY_NT_AUTHORITY, SECURITY_BATCH_RID, 0 } },
      { _T("SID_INTERACTIVE"), 
         { SID_REVISION, 1, SECURITY_NT_AUTHORITY, SECURITY_INTERACTIVE_RID, 0 } },
      { _T("SID_SERVICE"), 
         { SID_REVISION, 1, SECURITY_NT_AUTHORITY, SECURITY_SERVICE_RID, 0 } },
      { _T("SID_PROXY"), 
         { SID_REVISION, 1, SECURITY_NT_AUTHORITY, SECURITY_PROXY_RID, 0 } },
      { _T("SID_ENTERPRISE_CONTROLLERS"), 
         { SID_REVISION, 1, SECURITY_NT_AUTHORITY, SECURITY_ENTERPRISE_CONTROLLERS_RID, 0 } },
      { _T("SID_ANONYMOUS_LOGON"), 
         { SID_REVISION, 1, SECURITY_NT_AUTHORITY, SECURITY_ANONYMOUS_LOGON_RID, 0 } },
      { _T("SID_SELF"), 
         { SID_REVISION, 1, SECURITY_NT_AUTHORITY, SECURITY_PRINCIPAL_SELF_RID, 0 } },
      { _T("SID_AUTHENTICATED_USERS"), 
         { SID_REVISION, 1, SECURITY_NT_AUTHORITY, SECURITY_AUTHENTICATED_USER_RID, 0 } },
      { _T("SID_RESTRICTED"), 
         { SID_REVISION, 1, SECURITY_NT_AUTHORITY, SECURITY_RESTRICTED_CODE_RID, 0 } },
      { _T("SID_TERMINAL_SERVER"), 
         { SID_REVISION, 1, SECURITY_NT_AUTHORITY, SECURITY_TERMINAL_SERVER_RID, 0 } },
      { _T("SID_SYSTEM"), 
         { SID_REVISION, 1, SECURITY_NT_AUTHORITY, SECURITY_LOCAL_SYSTEM_RID, 0 } },
      // Well known SIDs in the BUILTIN domain
      { _T("SID_BUILTIN_ADMINISTRATORS"), 
         { SID_REVISION, 2, SECURITY_NT_AUTHORITY, SECURITY_BUILTIN_DOMAIN_RID, 
            DOMAIN_ALIAS_RID_ADMINS } },
      { _T("SID_BUILTIN_USERS"), 
         { SID_REVISION, 2, SECURITY_NT_AUTHORITY, SECURITY_BUILTIN_DOMAIN_RID, 
            DOMAIN_ALIAS_RID_USERS } },
      { _T("SID_BUILTIN_GUESTS"), 
         { SID_REVISION, 2, SECURITY_NT_AUTHORITY, SECURITY_BUILTIN_DOMAIN_RID, 
            DOMAIN_ALIAS_RID_GUESTS } },
      { _T("SID_BUILTIN_POWER_USERS"), 
         { SID_REVISION, 2, SECURITY_NT_AUTHORITY, SECURITY_BUILTIN_DOMAIN_RID, 
            DOMAIN_ALIAS_RID_POWER_USERS } },
      { _T("SID_BUILTIN_ACCOUNT_OPERATORS"), 
         { SID_REVISION, 2, SECURITY_NT_AUTHORITY, SECURITY_BUILTIN_DOMAIN_RID, 
            DOMAIN_ALIAS_RID_ACCOUNT_OPS } },
      { _T("SID_BUILTIN_SYSTEM_OPERATORS"), 
         { SID_REVISION, 2, SECURITY_NT_AUTHORITY, SECURITY_BUILTIN_DOMAIN_RID, 
            DOMAIN_ALIAS_RID_SYSTEM_OPS } },
      { _T("SID_BUILTIN_PRINT_OPERATORS"), 
         { SID_REVISION, 2, SECURITY_NT_AUTHORITY, SECURITY_BUILTIN_DOMAIN_RID, 
            DOMAIN_ALIAS_RID_PRINT_OPS } },
      { _T("SID_BUILTIN_BACKUP_OPERATORS"), 
         { SID_REVISION, 2, SECURITY_NT_AUTHORITY, SECURITY_BUILTIN_DOMAIN_RID, 
            DOMAIN_ALIAS_RID_BACKUP_OPS } },
      { _T("SID_BUILTIN_REPLICATOR"), 
         { SID_REVISION, 2, SECURITY_NT_AUTHORITY, SECURITY_BUILTIN_DOMAIN_RID, 
            DOMAIN_ALIAS_RID_REPLICATOR } },
      { _T("SID_BUILTIN_RAS_SERVERS"), 
         { SID_REVISION, 2, SECURITY_NT_AUTHORITY, SECURITY_BUILTIN_DOMAIN_RID, 
            DOMAIN_ALIAS_RID_RAS_SERVERS } },
      { _T("SID_BUILTIN_PRE_W2K_ACCESS"), 
         { SID_REVISION, 2, SECURITY_NT_AUTHORITY, SECURITY_BUILTIN_DOMAIN_RID, 
            DOMAIN_ALIAS_RID_PREW2KCOMPACCESS } },
      { NULL, { 0, 0, SECURITY_NULL_SID_AUTHORITY, 0 } },
   };

   for (SIDs const *pSID = &WellKnowns[0]; pSID->szName != NULL; ++pSID)
   {
      TCHAR          szUserName[260] = {0};
      DWORD          dwCB = _countof(szUserName);
      TCHAR          szDomain[260] = {0};
      DWORD          dwDO = _countof(szDomain);
      SID_NAME_USE   use;

      if (::LookupAccountSid(NULL, (PSID)&pSID->sid, szUserName, &dwCB, szDomain, &dwDO, &use))
      {
         TCHAR szValue[260] = {0};

         if (lstrlen(szDomain) > 0)
         {
            wsprintf(szValue, _T("%s\\%s"), szDomain, szUserName);
         }
         else
         {
            lstrcpy(szValue, szUserName);
         }

         ::MsiSetProperty(hInstall, pSID->szName, szValue);
      }
   }
   
   return ERROR_SUCCESS;
}